import { NonterminalKind } from '@nomicfoundation/slang/kinds/index.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { ABICoderPragma } from './ABICoderPragma.js';
import { ExperimentalPragma } from './ExperimentalPragma.js';
import { VersionPragma } from './VersionPragma.js';
export class Pragma {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.Pragma;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        switch (ast.variant.cst.kind) {
            case NonterminalKind.ABICoderPragma:
                this.variant = new ABICoderPragma(ast.variant, offsets[0]);
                break;
            case NonterminalKind.ExperimentalPragma:
                this.variant = new ExperimentalPragma(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.VersionPragma:
                this.variant = new VersionPragma(ast.variant, offsets[0]);
                break;
            default:
                throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
        }
        metadata = updateMetadata(metadata, [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return path.call(print, 'variant');
    }
}
//# sourceMappingURL=Pragma.js.map